﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// modified from https://unity3d.com/kr/learn/tutorials/topics/graphics/realtime-global-illumination-daynight-cycle

public class GlobalLightController : MonoBehaviour
{
	public Gradient nightDayColor;

	public float maxIntensity = 0.9f;
	public float minIntensity = 0.7f;

	public float maxAmbient = 1.0f;
	public float minAmbient = 0.7f;

	public float rotateSpeed = 2;

	private Light _light;

	// The time of this light in hour. This value is between 0 and 24.
	public float LightTime
	{
		get
		{
			float cos = Vector3.Dot(transform.forward, transform.parent.right);
			float sin = Vector3.Dot(transform.forward, transform.parent.forward);
			float angle = Mathf.Acos(cos) * 12 / Mathf.PI;
			if (sin >= 0)
				return 18 - angle;
			else if (angle <= 6)
				return 18 + angle;
			else
				return angle - 6;
		}
	}

	private void Start()
	{
		_light = GetComponent<Light>();
	}
	
	private void Update()
	{
		float cos = Vector3.Dot(transform.forward, transform.parent.right);
		float sin = Vector3.Dot(transform.forward, transform.parent.forward);
		float angle = Mathf.Acos(Mathf.Abs(cos));
		if (sin < 0)
			angle = 2 * Mathf.PI - angle;

		float range = maxIntensity - minIntensity;
		_light.intensity = minIntensity + range * Mathf.Clamp01(sin);

		range = maxAmbient - minAmbient;
		RenderSettings.ambientIntensity = minAmbient + range * Mathf.Clamp01(sin);

		_light.color = nightDayColor.Evaluate(angle / (2 * Mathf.PI));
		RenderSettings.ambientLight = _light.color;

		transform.Rotate(Vector3.up * rotateSpeed * Time.deltaTime);
	}
}
